@@ -1,12 +1,12 @@
 
 /*
- *  $Id: funcEval.c,v 1.48 2003/05/17 01:27:26 bkorb Exp $
+ *  $Id: funcEval.c,v 1.49 2004/02/02 03:31:50 bkorb Exp $
  *
  *  This module evaluates macro expressions.
  */
 
 /*
- *  AutoGen copyright 1992-2003 Bruce Korb
+ *  AutoGen copyright 1992-2004 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -195,7 +195,7 @@ evalExpression( ag_bool* pMustFree )
                 }
 
                 *pMustFree = AG_TRUE;
-                pzText = aprf( pT->pzTemplText + pMac->ozText, pDef->pzValue );
+                pzText = aprf(pT->pzTemplText + pMac->ozText, pDef->val.pzText);
             }
 
             else if (pMac->ozText != 0)
@@ -211,7 +211,7 @@ evalExpression( ag_bool* pMustFree )
                     return (char*)zNil;
                 }
 
-                pzText = pDef->pzValue;
+                pzText = pDef->val.pzText;
             }
 
             code &= EMIT_PRIMARY_TYPE;
@@ -228,7 +228,7 @@ evalExpression( ag_bool* pMustFree )
             *pMustFree = AG_FALSE;
         }
 
-        pzText = pDef->pzValue;
+        pzText = pDef->val.pzText;
         break;
 
     case EMIT_EXPRESSION:
